import { Http, Metadata } from '@zeedhi/core';
import { LoginInstance } from '@/services/LoginInstance';
import { AlertService, DialogService } from '@zeedhi/common';
import { LoginDialogService } from '@/services/LoginDialogService';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { ZdCollapseCard } from '@zeedhi/vuetify';

export class ConfigurationsController {
	private adConfigurationOnCreated: any;

	private parametersOnCreated: any;

	private isLoadingActiveDirectory = false;

	private toggleUseActiveDirectory = 'N';

	get denialToggleUseActiveDirectory() {
		if (this.isLoadingActiveDirectory) {
			return true;
		}
		return this.toggleUseActiveDirectory === 'N';
	}

	private isLoadingParameters = false;

	private toggleIpRestriction = 'S';

	private toggleUsePasswordControl = 'N';

	get denialToggleUsePasswordControl() {
		if (this.isLoadingParameters) {
			return true;
		}
		return this.toggleUsePasswordControl === 'N';
	}

	private isLoadingIpRestriction = false;

	private isDisabledIpRestrictionCollapseCard = 'N';

	get getIsDisabledIpRestrictionCollapseCard() {
		return this.isDisabledIpRestrictionCollapseCard === 'N';
	}

	private isVisibleCardAddIpRestriction = false;

	get denialIsVisibleCardAddIpRestriction() {
		return !this.isVisibleCardAddIpRestriction;
	}

	private isEditingCardAddIpRestriction = false;

	private idInputTextValue = '';

	private isActiveCheckBoxValue = 'N';

	get denialIsActiveCheckBoxValue() {
		if (this.isLoadingIpRestriction) {
			return true;
		}
		return !this.isEditingCardAddIpRestriction;
	}

	private ipv4InputTextValue = '';

	private ipv4InputTextDisabled = false;

	get getIpv4InputTextDisabled() {
		if (this.isLoadingIpRestriction) {
			return true;
		}
		return this.ipv4InputTextDisabled;
	}

	private ipv6InputTextValue = '';

	private ipv6InputTextDisabled = false;

	get getIpv6InputTextDisabled() {
		if (this.isLoadingIpRestriction) {
			return true;
		}
		return this.ipv6InputTextDisabled;
	}

	private descriptionInputText = '';

	private onCreatedLoginConfigGeneralDiv() {
		this.setAdConfigSetup();
		this.setParametersSetup();
	}

	private async setAdConfigSetup() {
		this.adConfigurationOnCreated = await Http.get('/ls_findADConfiguration');
		const formADMetadata = LoginInstance.getInstanceZdForm('formActiveDirectory');
		formADMetadata.value.inputPassword = this.adConfigurationOnCreated.data[0].ADM_PASSWORD;
		formADMetadata.value.inputUser = this.adConfigurationOnCreated.data[0].ADM_USER;
		formADMetadata.value.inputDomain = this.adConfigurationOnCreated.data[0].DOMAIN;
		formADMetadata.value.inputServer = this.adConfigurationOnCreated.data[0].SERVER;
		this.toggleUseActiveDirectory = this.adConfigurationOnCreated.data[0].USE_AD;
	}

	private async setParametersSetup() {
		this.parametersOnCreated = await Http.get('/organizationParameters');
		const formParametersMetadata = LoginInstance.getInstanceZdForm('formParameters');
		formParametersMetadata.value.inputValidationMask = this.parametersOnCreated.data.CDPICTSENHA;
		this.toggleUsePasswordControl = this.parametersOnCreated.data.IDUTCTRLSEN;
		formParametersMetadata.value.inputNecessaryReuseChanges = this.parametersOnCreated.data.NRALTSENHREP;
		formParametersMetadata.value.inputExpirationAlert = this.parametersOnCreated.data.NRDIASAVISO;
		formParametersMetadata.value.inputValidity = this.parametersOnCreated.data.NRDIASEXPSEN;
		formParametersMetadata.value.inputNumberOfPasswordAttempts = this.parametersOnCreated.data.NRERROSENHA;
		this.toggleIpRestriction = this.parametersOnCreated.data.USE_RESTRICTION;
		this.isDisabledIpRestrictionCollapseCard = this.parametersOnCreated.data.USE_RESTRICTION;
	}

	private clearFormActiveDirectory() {
		if (this.toggleUseActiveDirectory === 'N') {
			const formADMetadata = LoginInstance.getInstanceZdForm('formActiveDirectory');
			formADMetadata.value.inputPassword = '';
			formADMetadata.value.inputUser = '';
			formADMetadata.value.inputDomain = '';
			formADMetadata.value.inputServer = '';
			formADMetadata.resetValidation();
		}
	}

	private async clickSaveAdConfig() {
		this.isLoadingActiveDirectory = true;
		const formADMetadata = LoginInstance.getInstanceZdForm('formActiveDirectory');
		const sendObject = {
			ADM_PASSWORD: formADMetadata.value.inputPassword,
			ADM_USER: formADMetadata.value.inputUser,
			DOMAIN: formADMetadata.value.inputDomain,
			SERVER: formADMetadata.value.inputServer,
			USE_AD: this.toggleUseActiveDirectory,
		};
		try {
			if ((sendObject.DOMAIN && sendObject.SERVER) || this.toggleUseActiveDirectory === 'N') {
				await Http.post('/ls_findADConfiguration', sendObject);
				AlertService.show({
					name: 'alertAdError',
					text: 'Active Directory salvo com sucesso!',
					top: true,
					right: true,
					color: 'success',
					timeout: 2000,
					showDismiss: true,
				});
			} else {
				formADMetadata.validate();
			}
		} catch (e) {
			LoginDialogService.showErrorBackend(e);
		}
		this.isLoadingActiveDirectory = false;
	}

	private clearFormParameters() {
		if (this.toggleUsePasswordControl === 'N') {
			const formParametersMetadata = LoginInstance.getInstanceZdForm('formParameters');
			formParametersMetadata.value.inputValidationMask = '';
			formParametersMetadata.value.inputNecessaryReuseChanges = '';
			formParametersMetadata.value.inputExpirationAlert = '';
			formParametersMetadata.value.inputValidity = '';
			formParametersMetadata.value.inputNumberOfPasswordAttempts = '';
		}
	}

	private async clickSaveParameters() {
		this.isLoadingParameters = true;
		const formParametersMetadata = LoginInstance.getInstanceZdForm('formParameters');
		const sendObject = {
			CDPICTSENHA: formParametersMetadata.value.inputValidationMask,
			IDUTCTRLSEN: this.toggleUsePasswordControl,
			NRALTSENHREP: formParametersMetadata.value.inputNecessaryReuseChanges,
			NRDIASAVISO: formParametersMetadata.value.inputExpirationAlert,
			NRDIASEXPSEN: formParametersMetadata.value.inputValidity,
			NRERROSENHA: formParametersMetadata.value.inputNumberOfPasswordAttempts,
			USE_RESTRICTION: this.toggleIpRestriction,
		};
		try {
			await Http.post('/organizationParameters', sendObject);
			this.isDisabledIpRestrictionCollapseCard = this.toggleIpRestriction;
			const collapseCardIpRestrictionInstance = Metadata.getInstance<ZdCollapseCard>('collapseCardIpRestriction');
			collapseCardIpRestrictionInstance.expanded = false;
			AlertService.show({
				name: 'alertAdError',
				text: 'Parâmetros salvos com sucesso!',
				top: true,
				right: true,
				color: 'success',
				timeout: 2000,
				showDismiss: true,
			});
		} catch (e) {
			LoginDialogService.showErrorBackend(e);
		}
		this.isLoadingParameters = false;
	}

	private addClickIpGrid() {
		this.isVisibleCardAddIpRestriction = true;
		this.isActiveCheckBoxValue = 'S';
	}

	private onRowClickIpGrid(params: any) {
		this.isLoadingIpRestriction = true;
		this.isVisibleCardAddIpRestriction = true;
		this.isEditingCardAddIpRestriction = true;
		this.idInputTextValue = params.component.datasource.currentRowValue.ID;
		this.isActiveCheckBoxValue = params.component.datasource.currentRowValue.ACTIVE;
		this.ipv4InputTextValue = params.component.datasource.currentRowValue.IPV4;
		this.ipv6InputTextValue = params.component.datasource.currentRowValue.IPV6;
		this.descriptionInputText = params.component.datasource.currentRowValue.DESCRIPTION;
		this.checkIpv4Ipv6();
		this.isLoadingIpRestriction = false;
	}

	private checkIpv4Ipv6() {
		this.ipv4InputTextDisabled = false;
		this.ipv6InputTextDisabled = false;
		if (this.ipv4InputTextValue) {
			this.ipv6InputTextDisabled = true;
		} else if (this.ipv6InputTextValue) {
			this.ipv4InputTextDisabled = true;
		} else {
			this.ipv4InputTextDisabled = false;
			this.ipv6InputTextDisabled = false;
		}
	}

	private deleteEventDialog: any = {
		click: () => this.confirmDeleteIpRestriction(),
	};

	private deleteEventDialogClose: any = {
		click: () => this.closeDeleteDialog(),
	};

	private sendObjectDialog: any;

	private async deleteClickIpGrid(params: any) {
		this.isLoadingIpRestriction = true;
		this.sendObjectDialog = {
			ID: params.row.ID,
		};
		DialogService.show({
			name: 'errorDialogIpRestrictionDelete',
			type: 'error',
			title: 'Exclusão de IP',
			text: 'Você confirma a exclusão do IP?',
			buttons: [
				{ name: 'yes', component: 'ZdButton', outline: true, label: 'Sim', events: this.deleteEventDialog },
				{ name: 'no', component: 'ZdButton', label: 'Não', events: this.deleteEventDialogClose },
			],
		});
	}

	private async confirmDeleteIpRestriction() {
		try {
			await Http.post('/ls_OrganizationRestrictionByIp/delete', this.sendObjectDialog);
			AlertService.show({
				name: 'alertAdError',
				text: 'IP deletado comsucesso!',
				top: true,
				right: true,
				color: 'success',
				timeout: 2000,
				showDismiss: true,
			});
			this.tekGridUpdateIpConfig();
		} catch (e) {
			LoginDialogService.showErrorBackend(e);
		}
		this.closeDeleteDialog();
	}

	private closeDeleteDialog() {
		this.closeButtonCardAddIpRestriction();
		this.isLoadingIpRestriction = false;
		DialogService.hide();
	}

	private async clickSaveAddIpCongfig() {
		this.isLoadingIpRestriction = true;
		const sendObject = {
			ACTIVE: this.isActiveCheckBoxValue,
			DESCRIPTION: this.descriptionInputText,
			ID: this.idInputTextValue,
			IPV4: this.ipv4InputTextValue,
			IPV6: this.ipv6InputTextValue,
		};
		try {
			await Http.post('/ls_OrganizationRestrictionByIp', sendObject);
			AlertService.show({
				name: 'alertAdError',
				text: 'Restrição de IP salva com sucesso!',
				top: true,
				right: true,
				color: 'success',
				timeout: 2000,
				showDismiss: true,
			});
			this.closeButtonCardAddIpRestriction();
		} catch (e) {
			LoginDialogService.showErrorBackend(e);
		}
		this.tekGridUpdateIpConfig();
		this.isLoadingIpRestriction = false;
	}

	private closeButtonCardAddIpRestriction() {
		this.isVisibleCardAddIpRestriction = false;
		this.isEditingCardAddIpRestriction = false;
		this.ipv4InputTextDisabled = false;
		this.ipv6InputTextDisabled = false;
		this.idInputTextValue = '';
		this.isActiveCheckBoxValue = 'N';
		this.ipv4InputTextValue = '';
		this.ipv6InputTextValue = '';
		this.descriptionInputText = '';
	}

	private tekGridUpdateIpConfig() {
		const tekGridIpRestrictionInstance = Metadata.getInstance<TekGrid>('gridIpRestriction');
		tekGridIpRestrictionInstance.datasource.get();
	}
}
